@@ -378,7 +378,7 @@ struct OPENDHT_PUBLIC NodeExport {
 
     void msgpack_unpack(msgpack::object o);
 
-    OPENDHT_PUBLIC friend std::ostream& operator<< (std::ostream& s, const InfoHash& h);
+    OPENDHT_PUBLIC friend std::ostream& operator<< (std::ostream& s, const NodeExport& h);
 };
 
 }
