@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2014-2017 Savoir-faire Linux Inc.
+ *  Copyright (C) 2014-2018 Savoir-faire Linux Inc.
  *  Author(s) : Adrien Béraud <adrien.beraud@savoirfairelinux.com>
  *              Simon Désaulniers <simon.desaulniers@savoirfairelinux.com>
  *
@@ -39,14 +39,6 @@ constexpr std::chrono::minutes Dht::SEARCH_EXPIRE_TIME;
 constexpr std::chrono::seconds Dht::LISTEN_EXPIRE_TIME;
 constexpr std::chrono::seconds Dht::REANNOUNCE_MARGIN;
 
-void
-Dht::setLoggers(LogMethod error, LogMethod warn, LogMethod debug)
-{
-    DHT_LOG.DEBUG = debug;
-    DHT_LOG.WARN = warn;
-    DHT_LOG.ERR = error;
-}
-
 NodeStatus
 Dht::getStatus(sa_family_t af) const
 {
@@ -138,7 +130,7 @@ Dht::trySearchInsert(const Sp<Node>& node)
         auto& s = *it->second;
         if (s.insertNode(node, now)) {
             inserted = true;
-            scheduler.edit(s.nextSearchStep, s.getNextStepTime(now));
+            scheduler.edit(s.nextSearchStep, now);
         } else if (not s.expired and not s.done)
             break;
         ++it;
@@ -150,7 +142,7 @@ Dht::trySearchInsert(const Sp<Node>& node)
         auto& s = *it->second;
         if (s.insertNode(node, now)) {
             inserted = true;
-            scheduler.edit(s.nextSearchStep, s.getNextStepTime(now));
+            scheduler.edit(s.nextSearchStep, now);
         } else if (not s.expired and not s.done)
             break;
     }
@@ -225,6 +217,7 @@ Dht::searchNodeGetDone(const net::Request& req,
 {
     const auto& now = scheduler.time();
     if (auto sr = ws.lock()) {
+        sr->insertNode(req.node, now, answer.ntoken);
         if (auto srn = sr->getNode(req.node)) {
             /* all other get requests which are satisfied by this answer
                should not be sent anymore */
@@ -236,8 +229,12 @@ Dht::searchNodeGetDone(const net::Request& req,
                     srn->getStatus[q] = std::move(dummy_req);
                 }
             }
+            auto syncTime = srn->getSyncTime(scheduler.time());
+            if (srn->syncJob)
+                scheduler.edit(srn->syncJob, syncTime);
+            else
+                srn->syncJob = scheduler.add(syncTime, std::bind(&Dht::searchStep, this, sr));
         }
-        sr->insertNode(req.node, now, answer.ntoken);
         onGetValuesDone(req.node, answer, sr, query);
     }
 }
@@ -344,8 +341,8 @@ Dht::searchSendGetValues(Sp<Search> sr, SearchNode* pn, bool update)
             if (not n)
                 return nullptr;
 
-            DHT_LOG.w(sr->id, n->node->id, "[search %s] [node %s] sending 'find_node'",
-                    sr->id.toString().c_str(), n->node->toString().c_str());
+            /*DHT_LOG.d(sr->id, n->node->id, "[search %s] [node %s] sending 'find_node'",
+                    sr->id.toString().c_str(), n->node->toString().c_str());*/
             n->getStatus[query] = network_engine.sendFindNode(n->node,
                     sr->id,
                     -1,
@@ -358,8 +355,8 @@ Dht::searchSendGetValues(Sp<Search> sr, SearchNode* pn, bool update)
 
             if (query and not query->select.getSelection().empty()) {
                 /* The request contains a select. No need to paginate... */
-                DHT_LOG.w(sr->id, n->node->id, "[search %s] [node %s] sending 'get'",
-                        sr->id.toString().c_str(), n->node->toString().c_str());
+                /*DHT_LOG.d(sr->id, n->node->id, "[search %s] [node %s] sending 'get'",
+                        sr->id.toString().c_str(), n->node->toString().c_str());*/
                 n->getStatus[query] = network_engine.sendGetValues(n->node,
                         sr->id,
                         *query,
@@ -419,7 +416,7 @@ void Dht::searchSendAnnounceValue(const Sp<Search>& sr) {
                 const auto& now = scheduler.time();
                 if (not sn->isSynced(now)) {
                     /* Search is now unsynced. Let's call searchStep to sync again. */
-                    scheduler.edit(sr->nextSearchStep, sr->getNextStepTime(now));
+                    scheduler.edit(sr->nextSearchStep, now);
                     return;
                 }
                 for (auto& a : sr->announce) {
@@ -557,6 +554,8 @@ Dht::searchStep(Sp<Search> sr)
                         { /* on done */
                             if (auto sr = ws.lock()) {
                                 scheduler.edit(sr->nextSearchStep, scheduler.time());
+                                if (auto sn = sr->getNode(req.node))
+                                    scheduler.add(sn->getListenTime(query), std::bind(&Dht::searchStep, this, sr));
                                 onListenDone(req.node, answer, sr);
                             }
                         },
@@ -614,8 +613,8 @@ Dht::searchStep(Sp<Search> sr)
     /* dumpSearch(*sr, std::cout); */
 
     /* periodic searchStep scheduling. */
-    if (not sr->done)
-        scheduler.edit(sr->nextSearchStep, sr->getNextStepTime(now));
+    //if (not sr->done)
+    //    scheduler.edit(sr->nextSearchStep, now);
 }
 
 unsigned Dht::refill(Dht::Search& sr) {
@@ -703,7 +702,7 @@ Dht::search(const InfoHash& id, sa_family_t af, GetCallback gcb, QueryCallback q
 
     refill(*sr);
     if (sr->nextSearchStep)
-        scheduler.edit(sr->nextSearchStep, sr->getNextStepTime(scheduler.time()));
+        scheduler.edit(sr->nextSearchStep, scheduler.time());
     else
         sr->nextSearchStep = scheduler.add(scheduler.time(), std::bind(&Dht::searchStep, this, sr));
 
@@ -790,12 +789,12 @@ Dht::listenTo(const InfoHash& id, sa_family_t af, GetCallback cb, Value::Filter
     sr->done = false;
     auto token = ++sr->listener_token;
     sr->listeners.emplace(token, LocalListener{q, f, cb});
-    scheduler.edit(sr->nextSearchStep, sr->getNextStepTime(now));
+    scheduler.edit(sr->nextSearchStep, now);
     return token;
 }
 
 size_t
-Dht::listen(const InfoHash& id, GetCallback cb, Value::Filter&& f, Where&& where)
+Dht::listen(const InfoHash& id, GetCallback cb, Value::Filter f, Where where)
 {
     scheduler.syncTime();
 
@@ -1138,7 +1137,6 @@ Dht::cancelPut(const InfoHash& id, const Value::Id& vid)
 void
 Dht::storageChanged(const InfoHash& id, Storage& st, ValueStorage& v)
 {
-    DHT_LOG.d(id, "[store %s] changed", id.toString().c_str());
     if (not st.local_listeners.empty()) {
         DHT_LOG.d(id, "[store %s] %lu local listeners", id.toString().c_str(), st.local_listeners.size());
         std::vector<std::pair<GetCallback, std::vector<Sp<Value>>>> cbs;
@@ -1158,20 +1156,22 @@ Dht::storageChanged(const InfoHash& id, Storage& st, ValueStorage& v)
             cb.first(cb.second);
     }
 
-    DHT_LOG.d(id, "[store %s] %lu remote listeners", id.toString().c_str(), st.listeners.size());
-    for (const auto& node_listeners : st.listeners) {
-        for (const auto& l : node_listeners.second) {
-            auto f = l.second.query.where.getFilter();
-            if (f and not f(*v.data))
-                continue;
-            DHT_LOG.w(id, node_listeners.first->id, "[store %s] [node %s] sending update",
-                    id.toString().c_str(),
-                    node_listeners.first->toString().c_str());
-            std::vector<Sp<Value>> vals {};
-            vals.push_back(v.data);
-            Blob ntoken = makeToken(node_listeners.first->getAddr(), false);
-            network_engine.tellListener(node_listeners.first, l.second.sid, id, 0, ntoken, {}, {},
-                    std::move(vals), l.second.query);
+    if (not st.listeners.empty()) {
+        DHT_LOG.d(id, "[store %s] %lu remote listeners", id.toString().c_str(), st.listeners.size());
+        for (const auto& node_listeners : st.listeners) {
+            for (const auto& l : node_listeners.second) {
+                auto f = l.second.query.where.getFilter();
+                if (f and not f(*v.data))
+                    continue;
+                DHT_LOG.w(id, node_listeners.first->id, "[store %s] [node %s] sending update",
+                        id.toString().c_str(),
+                        node_listeners.first->toString().c_str());
+                std::vector<Sp<Value>> vals {};
+                vals.push_back(v.data);
+                Blob ntoken = makeToken(node_listeners.first->getAddr(), false);
+                network_engine.tellListener(node_listeners.first, l.first, id, 0, ntoken, {}, {},
+                        std::move(vals), l.second.query);
+            }
         }
     }
 }
@@ -1232,10 +1232,10 @@ Dht::storageAddListener(const InfoHash& id, const Sp<Node>& node, size_t socket_
                     buckets4.findClosestNodes(id, now, TARGET_NODES), buckets6.findClosestNodes(id, now, TARGET_NODES),
                     std::move(vals), query);
         }
-        node_listeners->second.emplace(socket_id, Listener {socket_id, now, std::forward<Query>(query)});
+        node_listeners->second.emplace(socket_id, Listener {now, std::forward<Query>(query)});
     }
     else
-        l->second.refresh(socket_id, now, std::forward<Query>(query));
+        l->second.refresh(now, std::forward<Query>(query));
 }
 
 void
@@ -1442,10 +1442,8 @@ Dht::dumpSearch(const Search& sr, std::ostream& out) const
         out << " [expired]";
     bool synced = sr.isSynced(now);
     out << (synced ? " [synced]" : " [not synced]");
-    if (synced && sr.isListening(now)) {
-        auto lt = sr.getListenTime(now);
-        out << " [listening, next in " << duration_cast<seconds>(lt-now).count() << " s]";
-    }
+    if (synced && sr.isListening(now))
+        out << " [listening]";
     out << std::endl;
 
     /*printing the queries*/
@@ -1657,7 +1655,7 @@ Dht::~Dht()
 
 Dht::Dht() : store(), scheduler(DHT_LOG), network_engine(DHT_LOG, scheduler) {}
 
-Dht::Dht(int s, int s6, Config config)
+Dht::Dht(const int& s, const int& s6, Config config)
     : myid(config.node_id ? config.node_id : InfoHash::getRandom()),
     is_bootstrap(config.is_bootstrap),
     maintain_storage(config.maintain_storage), store(), store_quota(),
@@ -2067,6 +2065,7 @@ Dht::onError(Sp<net::Request> req, net::DhtProtocolException e) {
                 n.token.clear();
                 n.last_get_reply = time_point::min();
                 searchSendGetValues(sr);
+                scheduler.edit(sr->nextSearchStep, scheduler.time());
                 break;
             }
         }
@@ -2121,8 +2120,6 @@ Dht::onGetValues(Sp<Node> node, const InfoHash& hash, want_t, const Query& query
     if (st != store.end() && not st->second.empty()) {
         answer.values = st->second.get(query.where.getFilter());
         DHT_LOG.d(hash, "[node %s] sending %u values", node->toString().c_str(), answer.values.size());
-    } else {
-        DHT_LOG.d(hash, "[node %s] sending nodes", node->toString().c_str());
     }
     return answer;
 }
@@ -2137,14 +2134,13 @@ void Dht::onGetValuesDone(const Sp<Node>& node,
         return;
     }
 
-    DHT_LOG.d(sr->id, "[search %s] [node %s] got reply to 'get' with %u nodes",
-            sr->id.toString().c_str(), node->toString().c_str(), a.nodes4.size()+a.nodes6.size());
+    /*DHT_LOG.d(sr->id, "[search %s] [node %s] got reply to 'get' with %u nodes",
+            sr->id.toString().c_str(), node->toString().c_str(), a.nodes4.size()+a.nodes6.size());*/
 
     if (not a.ntoken.empty()) {
         if (not a.values.empty() or not a.fields.empty()) {
-            DHT_LOG.d(sr->id, "[search %s IPv%c] found %u values",
-                    sr->id.toString().c_str(), sr->af == AF_INET ? '4' : '6',
-                    a.values.size());
+            DHT_LOG.d(sr->id, node->id, "[search %s] [node %s] found %u values",
+                      sr->id.toString().c_str(), node->toString().c_str(), a.values.size());
             for (auto& getp : sr->callbacks) { /* call all callbacks for this search */
                 auto& get = getp.second;
                 if (not (get.get_cb or get.query_cb) or
@@ -2229,7 +2225,7 @@ Dht::onListenDone(const Sp<Node>& node,
     if (not sr->done) {
         const auto& now = scheduler.time();
         searchSendGetValues(sr);
-        scheduler.edit(sr->nextSearchStep, sr->getNextStepTime(now));
+        scheduler.edit(sr->nextSearchStep, now);
     }
 }
 
