@@ -22,7 +22,7 @@
 #include "default_types.h"
 #include "securedht.h" // print certificate ID
 
-#if OPENDHT_PROXY_SERVER
+#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
 #include "base64.h"
 #endif //OPENDHT_PROXY_SERVER
 
@@ -172,7 +172,7 @@ Value::msgpack_unpack_body(const msgpack::object& o)
     }
 }
 
-#if OPENDHT_PROXY_SERVER
+#if OPENDHT_PROXY_SERVER || OPENDHT_PROXY_CLIENT
 Value::Value(Json::Value& json)
 {
    try {
