@@ -1,8 +1,9 @@
 /*
  *  Copyright (C) 2014-2017 Savoir-faire Linux Inc.
  *
- *  Author: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
- *          Simon Désaulniers <simon.desaulniers@savoirfairelinux.com>
+ *  Authors: Adrien Béraud <adrien.beraud@savoirfairelinux.com>
+ *           Simon Désaulniers <simon.desaulniers@savoirfairelinux.com>
+ *           Sébastien Blin <sebastien.blin@savoirfairelinux.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -59,14 +60,33 @@ void print_help() {
 
 #if OPENDHT_PROXY_SERVER
     std::cout << std::endl << "Operations with the proxy:" << std::endl
-              << "  pst [port]            Start the proxy interface on port." << std::endl
-              << "  psp [port]            Stop the proxy interface on port." << std::endl;
+#if OPENDHT_PUSH_NOTIFICATIONS
+              << "  pst [port] <pushServer> Start the proxy interface on port." << std::endl
+#else
+              << "  pst [port]              Start the proxy interface on port." << std::endl
+#endif // OPENDHT_PUSH_NOTIFICATIONS
+              << "  psp [port]              Stop the proxy interface on port." << std::endl;
 #endif //OPENDHT_PROXY_SERVER
 
+#if OPENDHT_PROXY_CLIENT
+    std::cout << std::endl << "Operations with the proxy:" << std::endl
+#if OPENDHT_PUSH_NOTIFICATIONS
+              << "  stt [server_address] <device_key> Start the proxy client." << std::endl
+#else
+              << "  stt [server_address]              Start the proxy client." << std::endl
+#endif // OPENDHT_PUSH_NOTIFICATIONS
+#if OPENDHT_PUSH_NOTIFICATIONS
+              << "  rs  [token]                       Resubscribe to opendht." << std::endl
+              << "  rp  [push_notification]           Inject a push notification in Opendht." << std::endl
+#endif // OPENDHT_PUSH_NOTIFICATIONS
+              << "  stp                               Stop the proxy client." << std::endl;
+#endif //OPENDHT_PROXY_CLIENT
+
     std::cout << std::endl << "Operations on the DHT:" << std::endl
               << "  b <ip:port>           Ping potential node at given IP address/port." << std::endl
               << "  g <key>               Get values at <key>." << std::endl
               << "  l <key>               Listen for value changes at <key>." << std::endl
+              << "  cl <key> <token>      Cancel listen for <token> and <key>." << std::endl
               << "  p <key> <str>         Put string value at <key>." << std::endl
               << "  pp <key> <str>        Put string value at <key> (persistent version)." << std::endl
               << "  s <key> <str>         Put string value at <key>, signed with our generated private key." << std::endl
@@ -79,7 +99,11 @@ void print_help() {
               << std::endl;
 }
 
-void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
+void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params
+#if OPENDHT_PROXY_SERVER
+    , std::map<in_port_t, std::unique_ptr<DhtProxyServer>>& proxies
+#endif
+)
 {
     print_node_info(dht, params);
     std::cout << " (type 'h' or 'help' for a list of possible commands)" << std::endl << std::endl;
@@ -90,12 +114,6 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
 #endif
 
     std::map<std::string, indexation::Pht> indexes;
-#if OPENDHT_PROXY_SERVER
-    std::map<in_port_t, std::unique_ptr<DhtProxyServer>> proxies;
-    if (params.proxyserver != 0) {
-        proxies.emplace(params.proxyserver, new DhtProxyServer(dht, params.proxyserver));
-    }
-#endif //OPENDHT_PROXY_SERVER
 
     while (true)
     {
@@ -105,7 +123,7 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
             break;
 
         std::istringstream iss(line);
-        std::string op, idstr, value, index, keystr;
+        std::string op, idstr, value, index, keystr, pushServer, deviceKey;
         iss >> op;
 
         if (op == "x" || op == "exit" || op == "quit") {
@@ -175,10 +193,18 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
         }
 #if OPENDHT_PROXY_SERVER
         else if (op == "pst") {
-            iss >> idstr;
+#if OPENDHT_PUSH_NOTIFICATIONS
+                iss >> idstr >> pushServer;
+#else
+                iss >> idstr;
+#endif // OPENDHT_PUSH_NOTIFICATIONS
             try {
                 unsigned int port = std::stoi(idstr);
-                proxies.emplace(port, new DhtProxyServer(dht, port));
+#if OPENDHT_PUSH_NOTIFICATIONS
+                proxies.emplace(port, std::unique_ptr<DhtProxyServer>(new DhtProxyServer(dht, port, pushServer)));
+#else
+                proxies.emplace(port, std::unique_ptr<DhtProxyServer>(new DhtProxyServer(dht, port)));
+#endif // OPENDHT_PUSH_NOTIFICATIONS
             } catch (...) { }
             continue;
         } else if (op == "psp") {
@@ -191,6 +217,29 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
             continue;
         }
 #endif //OPENDHT_PROXY_SERVER
+#if OPENDHT_PROXY_CLIENT
+        else if (op == "stt") {
+            dht->enableProxy(true);
+            continue;
+        } else if (op == "stp") {
+            dht->enableProxy(false);
+            continue;
+        }
+#if OPENDHT_PUSH_NOTIFICATIONS
+        else if (op == "rp") {
+            iss >> value;
+            dht->pushNotificationReceived({{"token", value}});
+            continue;
+        } else if (op == "re") {
+            iss >> value;
+            try {
+                unsigned token = std::stoul(value);
+                dht->resubscribe(token);
+            } catch (...) { }
+            continue;
+        }
+#endif // OPENDHT_PUSH_NOTIFICATIONS
+#endif //OPENDHT_PROXY_CLIENT
 
         if (op.empty())
             continue;
@@ -224,6 +273,10 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
                     indexes.emplace(index, Pht {index, std::move(ks), dht});
                 } catch (std::invalid_argument& e) { std::cout << e.what() << std::endl; }
             }
+        } else if (op == "cl") {
+            std::string hash, rem;
+            iss >> hash >> rem;
+            dht->cancelListen(dht::InfoHash(hash), std::stoul(rem));
         }
         else {
             // Dht syntax
@@ -277,12 +330,8 @@ void cmd_loop(std::shared_ptr<DhtRunner>& dht, dht_params& params)
                 std::cout << "\t" << *value << std::endl;
                 return true;
             }, {}, dht::Where {std::move(rem)});
-            std::cout << "Listening, token: " << token.get() << std::endl;
-        }
-        else if (op == "cl") {
-            std::string rem;
-            iss >> rem;
-            dht->cancelListen(id, std::stoul(rem));
+            auto t = token.get();
+            std::cout << "Listening, token: " << t << std::endl;
         }
         else if (op == "p") {
             std::string v;
@@ -418,6 +467,7 @@ main(int argc, char **argv)
             print_usage();
             return 0;
         }
+
         if (params.daemonize) {
             daemonize();
         } else if (params.service) {
@@ -427,10 +477,19 @@ main(int argc, char **argv)
         dht::crypto::Identity crt {};
         if (params.generate_identity) {
             auto ca_tmp = dht::crypto::generateEcIdentity("DHT Node CA");
-            crt = dht::crypto::generateEcIdentity("DHT Node", ca_tmp);
+            crt = dht::crypto::generateIdentity("DHT Node", ca_tmp);
         }
 
-        dht->run(params.port, crt, true, params.network);
+        dht::DhtRunner::Config config;
+        config.dht_config.node_config.network = params.network;
+        config.dht_config.id = crt;
+        config.threaded = true;
+        config.proxy_server = params.proxyclient;
+        config.push_node_id = "dhtnode";
+        if (not params.proxyclient.empty())
+            dht->setPushNotificationToken(params.devicekey);
+
+        dht->run(params.port, config);
 
         if (params.log) {
             if (params.syslog or (params.daemonize and params.logfile.empty()))
@@ -446,12 +505,27 @@ main(int argc, char **argv)
             dht->bootstrap(params.bootstrap.first.c_str(), params.bootstrap.second.c_str());
         }
 
-        if (params.daemonize or params.service) {
-            while (runner.wait());
-        } else {
-            cmd_loop(dht, params);
+#if OPENDHT_PROXY_SERVER
+        std::map<in_port_t, std::unique_ptr<DhtProxyServer>> proxies;
+#endif
+        if (params.proxyserver != 0) {
+#if OPENDHT_PROXY_SERVER
+            proxies.emplace(params.proxyserver, std::unique_ptr<DhtProxyServer>(new DhtProxyServer(dht, params.proxyserver, params.pushserver)));
+#else
+            std::cerr << "DHT proxy server requested but OpenDHT built without proxy server support." << std::endl;
+            exit(EXIT_FAILURE);
+#endif
         }
 
+        if (params.daemonize or params.service)
+            while (runner.wait());
+        else
+            cmd_loop(dht, params
+#if OPENDHT_PROXY_SERVER
+                , proxies
+#endif
+            );
+
     } catch(const std::exception&e) {
         std::cerr << std::endl <<  e.what() << std::endl;
     }
